using System;
using System.Collections;

namespace ClassLib
{
	/// <summary>
	/// Summary description for IndicatorCCI.
	/// </summary>
	public class IndicatorCCI : Indicator
	{
		private IndicatorJMA _jma;
		private ArrayList _candles;
		private double[] _cci;
		private int _period;		// time frame over which to apply cci

		public int Period {get{return _period;} set{_period = value;}}
		public double[] CCI {get{return _cci;} set{_cci = value;}}

		public IndicatorCCI()
		{
		}
		
		public IndicatorCCI(int p, ArrayList candles, IndicatorJMA jma)
		{
			_period = p;
			_candles = candles;
			_jma = jma;
		}

		public void CalculateCCI()
		{
			double[] meanDeviation = new double[_candles.Count];
			double[] typicalPrice = new double[_candles.Count];
			_cci = new double[_candles.Count];

			// initialise typical prices
			for (int i=0; i<_candles.Count; ++i)
				typicalPrice[i] = (((OHLC) _candles[i]).High + ((OHLC) _candles[i]).Low + ((OHLC) _candles[i]).Close) / 3;
			
			int startIdx = (_period < 30 ? 30 : _period);

			for (int i=startIdx; i<_candles.Count; ++i)
			{
				meanDeviation[i] = 0;
				for (int j=i; j>i-_period; --j)
				{
					meanDeviation[i] += Math.Abs((double) _jma.JMA[i] - typicalPrice[j]);
				}
				meanDeviation[i] /= _period;
			}
			
			for (int i=startIdx+1; i<_candles.Count; ++i)
				_cci[i] = (typicalPrice[i] - (double)_jma.JMA[i]) / (0.015 * meanDeviation[i]);
		}

		public double GetLastCCI()
		{
			return _cci[_cci.Length-1];
		}
	}
}
