using System;
using System.Collections;

namespace ClassLib
{
	/// <summary>
	/// Summary description for IndicatorJMA.
	/// </summary>
	public class IndicatorJMA
	{
		private ArrayList _candles;
		private ArrayList _jma;

		//---- buffers
		private double[] _JMAValueBuffer;
		private double[] _fC0Buffer;
		private double[] _fA8Buffer;
		private double[] _fC8Buffer;

		//---- input parameters
		private int _length;
		private int _phase;
		
		//---- bool flag
		private bool _initFlag;

		//---- temp buffers
		private double[] _list;
		private double[] _ring1;
		private double[] _ring2;
		private double[] _buffer;

		//---- integer vars
		private int _limitValue;
		private int _startValue;
		private int _loopParam;
		private int _loopCriteria;
		private int _cycleLimit;
		private int _highLimit;
		private int _counterA;
		private int _counterB;
		
		//---- double vars
		private double _cycleDelta;
		private double _lowDValue;
		private double _highDValue;
		private double _absValue;
		private double _paramA;
		private double _paramB;
		private double _phaseParam;
		private double _logParam;
		private double _JMAValue;
		private double _series;
		private double _sValue;
		private double _sqrtParam;
		private double _lengthDivider;

		//---- temporary int variables
		private int _s58;
		private int _s60;
		private int _s40;
		private int _s38;
		private int _s68;
		
		public int Length {get{return _length;} set{_length = value;}}
		public int Phase {get{return _phase;} set{_phase = value;}}
		public double[] JMAValueBuffer {get{return _JMAValueBuffer;} set{_JMAValueBuffer = value;}}
		public ArrayList JMA {get{return _jma;} set{_jma = value;}}

		public IndicatorJMA()
		{
		}

		public IndicatorJMA(ArrayList prices, int l, int p)
		{
			ArrayList tmp = new ArrayList();
			for (int i=prices.Count - 1; i>=0; --i)
				tmp.Add(prices[i]);
			_candles = tmp;
			_length = l;
			_phase = p;
			_jma = new ArrayList();
			/*for (int i=0; i<30; ++i)
				_jma.Add(0.0);//_jma.Insert(0, 0.0);*/
			Initialise();
		}

		public void CalculateJMA()
		{
			MainIteration();
		}

		private void Initialise()
		{
			double   lengthParam;

			_list = new double[128];
			_ring1 = new double[128];
			_ring2 = new double[11];
			_buffer = new double[62];

			_JMAValueBuffer = new double[_candles.Count];
			_fC0Buffer = new double[_candles.Count];
			_fA8Buffer = new double[_candles.Count];
			_fC8Buffer = new double[_candles.Count];
			
			_loopParam = 0;

			//---- initial part
			_limitValue = 63; 
			_startValue = 64;
			//----   
			for (int i=0; i<=_limitValue; i++) 
				_list[i] = -1000000;
			for (int i=_startValue; i<128; i++)
				_list[i] = 1000000;
			//----
			_initFlag  = true;

			if (Length < 1.0000000002)
				lengthParam = 0.0000000001;
			else
				lengthParam = (Length - 1) / 2.0;
			//----   
			if (Phase < -100)
				_phaseParam = 0.5;
			else if (Phase > 100)
				_phaseParam = 2.5;
			else
				_phaseParam = Phase / 100.0 + 1.5;
			//----   
			_logParam = Math.Log(Math.Sqrt(lengthParam)) / Math.Log (2.0);
			//----
			if (_logParam + 2.0 < 0)
				_logParam = 0;
			else
				_logParam = _logParam + 2.0; 
			//----
			_sqrtParam     = Math.Sqrt(lengthParam) * _logParam;
			lengthParam   = lengthParam * 0.9;
			_lengthDivider = lengthParam / (lengthParam + 2.0);
			//----  
		}

		private double MainIteration()
		{
			double dValue = 0;
			double powerValue;
			double squareValue;
			double JMATempValue = 0;
			double sqrtDivider;
			int limit = _candles.Count - 1/*Bars - counted_bars - 1*/;
			//---- main cycle
			for (int shift=limit; shift>=0; shift--)
			{
				//_loopParam = 30;
				_series = ((OHLC) _candles[shift]).Close;
				
				if (_loopParam < 61)
				{
					_loopParam++;
					_buffer[_loopParam] = _series;
				}
				if (_loopParam > 30)
				{
					if (_initFlag) 
					{ 
						_initFlag = false;
						int diffFlag = 0; 

						for (int i=1; i<=29; i++)
						{
							if (_buffer[i + 1] != _buffer[i])
								diffFlag = 1;
						}
						
						_highLimit = diffFlag * 30;
             
						if (_highLimit == 0)
						{
							_paramB = _series;
						}
						else
						{
							_paramB = _buffer[1];
						}
             
						_paramA = _paramB; 
						if (_highLimit > 29)
						{
							_highLimit = 29; 
						}
					} 
					else 
						_highLimit = 0;
					
					//---- big cycle
					for (int i=_highLimit; i>=0; i--) 
					{ 
						
						if (i == 0)
						{
							_sValue = _series;
						}
						else
						{
							_sValue = _buffer[31 - i]; 
						}
	    
						if (Math.Abs(_sValue - _paramA) > Math.Abs(_sValue - _paramB))
						{
							_absValue = Math.Abs(_sValue - _paramA);
						}
						else
						{
							_absValue = Math.Abs(_sValue - _paramB); 
						}
						dValue = _absValue + 0.0000000001; //1.0e-10;
				
						if (_counterA <= 1)
						{
							_counterA = 127;
						}
						else
						{
							_counterA--;
						}
						
						if (_counterB <= 1)
						{
							_counterB = 10; 
						}
						else
						{
							_counterB--; 
						}
						
						if (_cycleLimit < 128)
						{
								_cycleLimit++; 
						}
						_cycleDelta += (dValue - _ring2[_counterB]);
						_ring2[_counterB] = dValue; 
						
						if (_cycleLimit > 10)
						{
							_highDValue = _cycleDelta / 10.0;
						}
						else
						{
							_highDValue = _cycleDelta / _cycleLimit; 
						}
						
						if (_cycleLimit > 127)
						{
							dValue = _ring1[_counterA];
							_ring1[_counterA] = _highDValue;
							_s68 = 64;
							_s58 = _s68;
							while (_s68 > 1)
							{
								if (_list[_s58] < dValue)
								{
									_s68 = (int) (_s68 / 2.0);
									_s58 += _s68;
								}
								else if (_list[_s58] <= dValue)
								{
									_s68 = 1;
								}
								else
								{
									_s68 = (int) (_s68 / 2.0);
									_s58 -= _s68;
								}
							}
						}

						else
						{
							_ring1[_counterA] = _highDValue;
							if ((_limitValue + _startValue) > 127)
							{
								_startValue--;
								_s58 = _startValue;
							}
							else
							{
								_limitValue++;
								_s58 = _limitValue;
							}
							if (_limitValue > 96)
							{
								_s38 = 96;
							}
							else
							{
								_s38 = _limitValue;
							}
							if (_startValue < 32)
							{
								_s40 = 32;
							}
							else
							{
								_s40 = _startValue;
							}
						}
						//----		      
						_s68 = 64; 
						_s60 = _s68; 
						while (_s68 > 1) 
						{
							if (_list[_s60] >= _highDValue)
							{
								if (_list[_s60 - 1] <= _highDValue)
								{
									_s68 = 1;
								}
								else
								{
									_s68 = (int) (_s68 / 2.0);
									_s60 -= _s68;
								}
							}
							else
							{
								_s68 = (int) (_s68 / 2.0);
								_s60 += _s68;
							}
							if ((_s60 == 127) && (_highDValue > _list[127]))
							{
								_s60 = 128;
							}
						}
						
						if (_cycleLimit > 127) 
						{
							if (_s58 >= _s60) 
							{
								if (((_s38 + 1) > _s60) && ((_s40 - 1) < _s60)) 
									_lowDValue += _highDValue; 
								else if ((_s40 > _s60) && ((_s40 - 1) < _s58)) 
									_lowDValue += _list [_s40-1]; 
							}
							else if (_s40 >= _s60) 
							{
								if (((_s38 + 1) < _s60) && ((_s38 + 1) > _s58)) 
									_lowDValue += _list[_s38+1];  
							}
							else if ((_s38 + 2) > _s60) 
								_lowDValue += _highDValue; 
							else if (((_s38 + 1) < _s60) && ((_s38 + 1) > _s58)) 
								_lowDValue += _list[_s38+1]; 
			
							if (_s58 > _s60) 
							{
								if (((_s40 - 1) < _s58) && ((_s38 + 1) > _s58)) 
									_lowDValue -= _list[_s58]; 
								else if ((_s38 < _s58) && ((_s38 + 1) > _s60)) 
									_lowDValue -= _list [_s38]; 
							}
							else 
							{
								if (((_s38 + 1) > _s58) && ((_s40 - 1) < _s58)) 
									_lowDValue -= _list[_s58]; 
								else if ((_s40 > _s58) && (_s40 < _s60)) 
									_lowDValue -= _list[_s40]; 
							}
						}
						
						if (_s58 <= _s60) 
						{
							if (_s58 >= _s60)
								_list[_s60] = _highDValue;
							else 
							{
								for (int j=_s58+1; j<=(_s60-1); ++j) 
								{
									_list[j-1] = _list[j]; 
								}
								_list[_s60-1] = _highDValue; 
							}
						} 
						else 
						{
							for (int j=_s58-1; j>=_s60; j--) 
							{
								_list[j+1] = _list[j]; 
							}
							_list[_s60] = _highDValue; 
						}
			
						if (_cycleLimit <= 127) 
						{
							_lowDValue = 0; 
							for (int j = _s40; j <= _s38; j++) 
							{
								_lowDValue += _list[j]; 
							}
						}
						
						//----			    
						if ((_loopCriteria + 1) > 31)
							_loopCriteria = 31;
						else
							_loopCriteria++;

						sqrtDivider = _sqrtParam / (_sqrtParam + 1.0);
						
						if (_loopCriteria <= 30)
						{
							if (_sValue - _paramA > 0)
							{
								_paramA = _sValue;
							}
							else
							{
								_paramA = _sValue - (_sValue - _paramA) * sqrtDivider;
							}
							if (_sValue - _paramB < 0)
							{
								_paramB = _sValue;
							}
							else
							{
								_paramB = _sValue - (_sValue - _paramB) * sqrtDivider;
							}
							JMATempValue = _series;
							
							if (_loopCriteria == 30) 
							{ 
								_fC0Buffer[shift] = _series;
								int intPart;
				      
								if (Math.Ceiling(_sqrtParam) >= 1)
									intPart = (int) Math.Ceiling(_sqrtParam);
								else
									intPart = 1;
								int leftInt = IntPortion(intPart);
								if (Math.Floor(_sqrtParam) >= 1)
									intPart = (int) Math.Floor(_sqrtParam);
								else
									intPart = 1;
								int rightPart = IntPortion(intPart);
				     
								if (leftInt == rightPart)
									dValue = 1.0; 
								else 
									dValue = (_sqrtParam - rightPart) / (leftInt - rightPart);
			     
								int upShift, dnShift;
								if (rightPart <= 29)
									upShift = rightPart;
								else
									upShift = 29; 
								if (leftInt <= 29)
									dnShift = leftInt;
								else
									dnShift = 29; 
								_fA8Buffer[shift] = (_series - _buffer[_loopParam - upShift]) * (1 - dValue) / rightPart + (_series - _buffer[_loopParam - dnShift]) * dValue / leftInt;
							}
						}
							
						else 
						{
							dValue = _lowDValue / (_s38 - _s40 + 1);
							if (0.5 <= _logParam - 2.0)
								powerValue = _logParam - 2.0;
							else
								powerValue = 0.5;
               
							if (_logParam >= Math.Pow(_absValue/dValue, powerValue))
								dValue = Math.Pow(_absValue/dValue, powerValue);
							else
								dValue = _logParam; 
							if (dValue < 1)
								dValue = 1;
				    
							powerValue = Math.Pow(sqrtDivider, Math.Sqrt(dValue));
							if (_sValue - _paramA > 0)
								_paramA = _sValue;
							else
								_paramA = _sValue - (_sValue - _paramA) * powerValue;
							if (_sValue - _paramB < 0)
								_paramB = _sValue;
							else
								_paramB = _sValue - (_sValue - _paramB) * powerValue;
						}
					}
				
					// ---- end of big cycle                  			   
					if (_loopCriteria > 30) 
					{
						JMATempValue = _JMAValueBuffer[shift + 1];
						powerValue   = Math.Pow(_lengthDivider, dValue);
						squareValue  = Math.Pow(powerValue, 2);
                         
						_fC0Buffer[shift] = (1 - powerValue) * _series + powerValue * _fC0Buffer[shift + 1];
						_fC8Buffer[shift] = (_series - _fC0Buffer[shift]) * (1 - _lengthDivider) + _lengthDivider * _fC8Buffer[shift + 1];
            
						_fA8Buffer[shift] = (_phaseParam * _fC8Buffer[shift] + _fC0Buffer[shift] - JMATempValue) * 
							(powerValue * (-2.0) + squareValue + 1) + squareValue * _fA8Buffer[shift + 1];  
						JMATempValue += _fA8Buffer[shift]; 
					}
					_JMAValue = JMATempValue;
				}
				if (_loopParam <= 30)
					_JMAValue = 0;
				_JMAValueBuffer[shift] = _JMAValue;
				_jma.Add(_JMAValue);//_jma.Insert(0, _JMAValue);
				//---- End of main cycle
			}
			return 0.0;
		}

		private int IntPortion(double param) 
		{
			if (param > 0)
				return (int) Math.Floor(param);
			if (param < 0)
				return (int) Math.Ceiling(param);
			return 0;
		}
	}

}
